from typing import Dict, Optional

from clipped.utils.dates import to_timestamp
from clipped.utils.enums import get_enum_value
from clipped.utils.http import absolute_uri, add_notification_referrer_param
from vents.notifiers import NotificationSpec as _NotificationSpec

from polyaxon import settings
from polyaxon.connections import CONNECTION_CONFIG
from polyaxon.lifecycle import StatusColor, V1StatusCondition
from polyaxon.schemas.base import BaseSchemaModel
from polyaxon.utils.urls_utils import get_run_url


class NotificationSpec(_NotificationSpec, BaseSchemaModel):
    @staticmethod
    def load_from_data(
        kind: str,
        owner: str,
        project: str,
        uuid: str,
        name: str,
        status: str,
        wait_time: str,
        duration: str,
        condition: V1StatusCondition,
        inputs: Dict,
        outputs: Dict,
    ) -> "NotificationSpec":
        def get_details() -> str:
            details = "{} ({})".format(name, uuid) if name else uuid
            details = "Run: {}\n".format(details)
            if kind:
                details = "Kind: `{}`\n".format(get_enum_value(kind))
            details += "Status: `{}`\n".format(status)
            if condition.reason:
                details += "Reason: `{}`\n".format(condition.reason)
            if condition.message:
                details += "Message: `{}`\n".format(condition.message)
            details += "Transition time: `{}`\n".format(condition.last_transition_time)
            if wait_time:
                details += "Wait time: `{}`\n".format(wait_time)
            if duration:
                details += "Duration: `{}`\n".format(duration)
            if inputs:
                details += "Inputs: `{}`\n".format(inputs)
            if outputs:
                details += "Outputs: `{}`\n".format(outputs)

            return details

        def get_title() -> str:
            title = "{} ({})".format(name, uuid) if name else uuid
            title += " Status: {}\n".format(status)
            return title

        def get_color() -> str:
            return StatusColor.get_color(condition.type)

        def get_url() -> Optional[str]:
            if not (owner and project and uuid):
                return
            uri = get_run_url(owner=owner, project_name=project, run_uuid=uuid)
            uri = "ui{}".format(uri)
            url = absolute_uri(host=settings.CLIENT_CONFIG.host, url=uri)
            url = add_notification_referrer_param(
                url,
                provider=CONNECTION_CONFIG.project_name,
                is_absolute=False,
            )
            return url

        return NotificationSpec(
            context={
                "owner": owner,
                "project": project,
                "uuid": uuid,
                "name": name,
            },
            fallback=condition.type,
            title=get_title(),
            details=get_details(),
            description=condition.reason,
            url=get_url(),
            color=get_color(),
            ts=to_timestamp(condition.last_transition_time),
        )



from typing import List, Optional, Union

from clipped.types.ref_or_obj import BoolOrRef, RefField
from pydantic import Field, StrictStr

from polyaxon.auxiliaries.sidecar import V1PolyaxonSidecarContainer
from polyaxon.polyflow.notifications import V1Notification
from polyaxon.schemas.base import BaseSchemaModel


class V1Plugins(BaseSchemaModel):
    """Plugins section provides a way to customize extra Polyaxon utilities.

    By default, Polyaxon injects some information for example an auth context
    and triggers some mechanisms for collecting logs and outputs.

    Plugins section provides more control to the end user to enable/disable some of these utilities.

    Args:
        auth: bool, optional, default: True
        docker: bool, optional, default: False
        shm: bool, optional, default: True
        mount_artifacts_store: bool, optional, default: True
        collect_artifacts: bool, optional, default: True
        collect_logs: bool, optional, default: True
        collect_resources: bool, optional, default: True
        auto_resume: bool, optional, default: True
        log_level: str, optional
        sidecar: V1PolyaxonSidecarContainer, optional

    ## YAML usage

    ```yaml
    >>> plugins:
    >>>   auth:
    >>>   docker:
    >>>   shm:
    >>>   mountArtifactsStore:
    >>>   collectArtifacts:
    >>>   collectLogs:
    >>>   collectResources:
    >>>   autoResume:
    >>>   externalHost:
    >>>   logLevel:
    >>>   sidecar:
    ```

    ## Python usage

    ```python
    >>> from polyaxon.polyflow import V1Plugins
    >>> plugins = V1Plugins(
    >>>     auth=False,
    >>>     docker=True,
    >>>     shm=True.
    >>>     mount_artifacts_store=True,
    >>>     collect_artifacts=False,
    >>>     collect_logs=False,
    >>>     collect_resources=False
    >>>     auto_resume=False,
    >>>     external_host=False,
    >>>     log_level="INFO",
    >>> )
    ```

    ## Fields

    ### auth

    <blockquote class="light">
    This plugin is enabled by default in Polyaxon deployments with user management.
    </blockquote>

    By default, Polyaxon will create an auth context for each operation, this removes the overhead
    to think about how you can pass tokens to your runs,
    or the need to create secrets to load the token from during the run time.

    The auth context that Polyaxon provides is not only specific to the user who
    executed the run, but it also impersonates similar user access rights, it has the same scopes
    and restrictions the user usually has within the context of the project
    managing the run.
    This is important to make sure the API calls made during the run
    by the user's code have the right access to the resources requested.

    Oftentimes, users might not need to use an authenticated client inside their containers,
    in that case they can disable this plugin.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   auth: false
    ```

    ### docker

    <blockquote class="light">This plugin is disabled by default.</blockquote>

    This plugin exposes a docker socket volume to your run container.

    N.B. use this plugin carefully, you might also need to check with your devops
    team before using it, it requires docker.sock of the host to be mounted
    which is often rejected by OPA.

    To enable this plugin:

    ```yaml
    >>> plugins:
    >>>   docker: true
    ```

    ### shm

    <blockquote class="light">This plugin is enabled by default.</blockquote>

    This plugin mounts an tmpfs volume to /dev/shm.
    This will set /dev/shm size to half of the RAM of node.
    By default, /dev/shm is very small, only 64MB.
    Some experiments/jobs will fail due to the lack of shared memory,
    such as some experiments running on Pytorch.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   shm: false
    ```

    ### mountArtifactsStore

    <blockquote class="light">This plugin is disabled by default.</blockquote>

    this plugin allows to request the default artifacts store and mount it to the main container
    without adding the connection reference name to the `connections` section.

    This is usually very useful than setting the `connections` section as it make the component
    more generic and will not break if the artifacts store name changes.

    ### collectArtifacts

    <blockquote class="light">This plugin is enabled by default.</blockquote>

    By default, Polyaxon will collect all artifacts and outputs that you share in the
    `plx-context/artifacts/run-uuid/outputs` to the default artifacts store
    that you configured during the deployment.

    This plugin is important if you want to have an agnostic code to the
    type of storage backend your are using, by changing the environment variable
    you can test your code with `tmp` file locally and the artifacts path in production.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   collectArtifacts: false
    ```

    Sometimes you might want to access the artifacts path in your polyaxonfile,
    Polyaxon expose a [context](/docs/core//context/) that get resolved during
    the compilation time, you can just use
    "{{run_artifacts_path}}" global variable and it will be resolved automatically.

    Example:

    ```yaml
    >>> run:
    >>>   kind: job
    >>>   container:
    >>>     command: "cp /some/know/path/file {{run_artifacts_path}}/file"

    ```

    For more information about the context, please check [context](/docs/core/context/)

    ### collectLogs

    <blockquote class="light">This plugin is enabled by default.</blockquote>

    By default, Polyaxon will collect all logs related to your runs before deleting
    the resource on the clusters. This ensures that your cluster(s) are kept clean and no resources
    are actively putting pressure on the API server.

    Sometimes you might want to avoid collecting logs for some runs, for example test or debug jobs.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   collectLogs: false
    ```

    ### collectResources

    <blockquote class="light">This plugin is enabled by default.</blockquote>

    By default, Polyaxon will collect all Mem/CPU/GPU resources
    for your runs that use the python client.

    Sometimes you might want to avoid collecting this information for some runs,
    for example test or debug jobs.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   collectResources: false
    ```

    ### autoResume

    <blockquote class="light">This plugin is enabled by default.</blockquote>

    By default, Polyaxon will resume from collecting metrics/outputs/artifacts
    if a run fails and retries or if the user resume a run.

    To disable this plugin:

    ```yaml
    >>> plugins:
    >>>   autoResume: false
    ```

    ### externalHost
    <blockquote class="light">Default is False.</blockquote>
    In some edge cases where the auxiliaries and/or the main container cannot reach the API/Streams
    services via the internal networking interface, you can enable this flag to tell Polyaxon
    resolve the external host instead of the default behavior with the in-cluster host.

    ```yaml
    >>> plugins:
    >>>   externalHost: true
    ```

    ### logLevel

    <blockquote class="light">Default is None.</blockquote>

    If you want to control the log level of your runs in a similar way locally and on the cluster,
    you can either use env vars or this plugin to share the same log level with all containers
    running in your operation.

    ```yaml
    >>> plugins:
    >>>   logLevel: warning
    ```

    ### sidecar

    <blockquote class="light">Default is None.</blockquote>

    To override the default global sidecar configuration.

    ```yaml
    >>> plugins:
    >>>   sidecar:
    >>>     syncInterval: 60
    ```
    """

    _IDENTIFIER = "plugins"

    auth: Optional[BoolOrRef]
    docker: Optional[BoolOrRef]
    shm: Optional[BoolOrRef]
    mount_artifacts_store: Optional[BoolOrRef] = Field(alias="mountArtifactsStore")
    collect_artifacts: Optional[BoolOrRef] = Field(alias="collectArtifacts")
    collect_logs: Optional[BoolOrRef] = Field(alias="collectLogs")
    collect_resources: Optional[BoolOrRef] = Field(alias="collectResources")
    sync_statuses: Optional[BoolOrRef] = Field(alias="syncStatuses")
    auto_resume: Optional[BoolOrRef] = Field(alias="autoResume")
    log_level: Optional[StrictStr] = Field(alias="logLevel")
    external_host: Optional[BoolOrRef] = Field(alias="externalHost")
    sidecar: Optional[Union[V1PolyaxonSidecarContainer, RefField]]
    notifications: Optional[Union[List[V1Notification], RefField]]

    @classmethod
    def get_or_create(
        cls, config: Optional["V1Plugins"], auth: bool = False
    ) -> "V1Plugins":
        if not config:
            config = cls(auth=auth)
        else:
            config = config.copy()
        config.set_auth(default=auth)
        config.set_docker()
        config.set_shm()
        config.set_mount_artifacts_store()
        config.set_collect_artifacts()
        config.set_collect_logs()
        config.set_collect_resources()
        config.set_sync_statuses()
        config.set_auto_resume()
        config.set_external_host()
        return config

    @staticmethod
    def no_api():
        from polyaxon import settings

        return settings.CLIENT_CONFIG.no_api

    def set_auth(self, default: bool = False):
        if self.no_api():
            self.auth = False
        elif self.auth is None:
            self.auth = default

    def set_docker(self, default: bool = False):
        if self.docker is None:
            self.docker = default

    def set_shm(self, default: bool = True):
        if self.shm is None:
            self.shm = default

    def set_mount_artifacts_store(self, default: bool = False):
        if self.mount_artifacts_store is None:
            self.mount_artifacts_store = default

    def set_collect_artifacts(self, default: bool = True):
        if self.no_api():
            self.collect_artifacts = False
        elif self.collect_artifacts is None:
            self.collect_artifacts = default

    def set_collect_logs(self, default: bool = True):
        if self.no_api():
            self.collect_logs = False
        elif self.collect_logs is None:
            self.collect_logs = default

    def set_collect_resources(self, default: bool = True):
        if self.no_api():
            self.collect_resources = False
        elif self.collect_resources is None:
            self.collect_resources = default

    def set_sync_statuses(self, default: bool = True):
        if self.no_api():
            self.sync_statuses = False
        elif self.sync_statuses is None:
            self.sync_statuses = default

    def set_auto_resume(self, default: bool = True):
        if self.no_api():
            self.auto_resume = False
        elif self.auto_resume is None:
            self.auto_resume = default

    def set_external_host(self, default: bool = False):
        if self.external_host is None:
            self.external_host = default



from pydantic import validate_arguments

from polyaxon.schemas.api.authentication import V1Credentials
from polyaxon.schemas.responses.v1_auth import V1Auth
from polyaxon.schemas.responses.v1_password_change import V1PasswordChange
from polyaxon.schemas.responses.v1_trial_start import V1TrialStart
from polyaxon.schemas.responses.v1_user_email import V1UserEmail
from polyaxon.schemas.responses.v1_user_singup import V1UserSingup
from polyaxon.sdk.base_api import BaseApi
from polyaxon.sdk.exceptions import ApiTypeError


class AuthV1Api(BaseApi):
    @validate_arguments
    def change_password(self, body: V1PasswordChange, **kwargs) -> None:
        """Change password

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.change_password(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1PasswordChange
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.change_password_with_http_info(body, **kwargs)

    @validate_arguments
    def change_password_with_http_info(self, body: V1PasswordChange, **kwargs):
        """Change password

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.change_password_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1PasswordChange
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method change_password" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            "/api/v1/auth/change-password",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def login(self, body: V1Credentials, **kwargs) -> V1Auth:  # noqa: E501
        """Login  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.login(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1Credentials
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1Auth
        """
        kwargs["_return_http_data_only"] = True
        return self.login_with_http_info(body, **kwargs)  # noqa: E501

    @validate_arguments
    def login_with_http_info(self, body: V1Credentials, **kwargs):  # noqa: E501
        """Login  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.login_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1Credentials
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1Auth, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'" " to method login" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {
            "200": "V1Auth",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/auth/token",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def reset_password(self, body: V1UserEmail, **kwargs) -> None:  # noqa: E501
        """Reset password  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_password(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1UserEmail
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.reset_password_with_http_info(body, **kwargs)  # noqa: E501

    @validate_arguments
    def reset_password_with_http_info(self, body: V1UserEmail, **kwargs):  # noqa: E501
        """Reset password  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_password_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1UserEmail
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method reset_password" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            "/api/v1/auth/reset-password",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def reset_password_confirm(
        self, body: V1PasswordChange, **kwargs
    ) -> V1Auth:  # noqa: E501
        """Reset password confirm  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_password_confirm(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1PasswordChange
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1Auth
        """
        kwargs["_return_http_data_only"] = True
        return self.reset_password_confirm_with_http_info(body, **kwargs)  # noqa: E501

    @validate_arguments
    def reset_password_confirm_with_http_info(
        self, body: V1PasswordChange, **kwargs
    ):  # noqa: E501
        """Reset password confirm  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.reset_password_confirm_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1PasswordChange
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1Auth, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method reset_password_confirm" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {
            "200": "V1Auth",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/auth/reset-password-confirm",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def signup(self, body: V1UserSingup, **kwargs) -> V1Auth:  # noqa: E501
        """Signup  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.signup(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1UserSingup
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: V1Auth
        """
        kwargs["_return_http_data_only"] = True
        return self.signup_with_http_info(body, **kwargs)  # noqa: E501

    @validate_arguments
    def signup_with_http_info(self, body: V1UserSingup, **kwargs):  # noqa: E501
        """Signup  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.signup_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1UserSingup
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(V1Auth, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'" " to method signup" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {
            "200": "V1Auth",
            "204": "object",
            "403": "object",
            "404": "object",
        }

        return self.api_client.call_api(
            "/api/v1/auth/signup",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def trial(self, body: V1TrialStart, **kwargs) -> None:  # noqa: E501
        """Trial Start  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.trial(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1TrialStart
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        return self.trial_with_http_info(body, **kwargs)  # noqa: E501

    @validate_arguments
    def trial_with_http_info(self, body: V1TrialStart, **kwargs):  # noqa: E501
        """Trial Start  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.trial_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param body: (required)
        :type body: V1TrialStart
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["body"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'" " to method trial" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None
        if _params["body"]:
            _body_params = _params["body"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["ApiKey"]  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            "/api/v1/auth/trial",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )



import copy

from typing import Dict, Optional

from polyaxon.compiler.contexts.base import BaseContextsManager
from polyaxon.connections import V1Connection
from polyaxon.polyflow import V1CompiledOperation, V1KFReplica, V1PytorchJob


class PytorchJobContextsManager(BaseContextsManager):
    @classmethod
    def resolve(
        cls,
        namespace: str,
        owner_name: str,
        project_name: str,
        run_uuid: str,
        contexts: Dict,
        compiled_operation: V1CompiledOperation,
        connection_by_names: Dict[str, V1Connection],
    ) -> Dict:
        contexts["init"] = {}
        contexts["connections"] = {}
        job = compiled_operation.run  # type: V1PytorchJob

        def _get_replica(replica: Optional[V1KFReplica]) -> Dict:
            if not replica:
                return contexts
            return cls._resolver_replica(
                contexts={"globals": copy.copy(contexts["globals"])},
                init=replica.init,
                connections=replica.connections,
                connection_by_names=connection_by_names,
            )

        return {
            "master": _get_replica(job.master),
            "worker": _get_replica(job.worker),
        }



from typing import Dict, List, Optional, Union

from clipped.utils.lists import to_list
from clipped.utils.sanitizers import sanitize_value

from polyaxon.containers.names import sanitize_container_name
from polyaxon.k8s import k8s_schemas
from polyaxon.runner.converter import BaseConverter
from polyaxon.runner.converter.common.containers import sanitize_container_command_args


class ContainerMixin(BaseConverter):
    @classmethod
    def _patch_container(
        cls,
        container: k8s_schemas.V1Container,
        name: Optional[str] = None,
        command: Optional[List[str]] = None,
        args: Optional[List[str]] = None,
        image: Optional[str] = None,
        image_pull_policy: Optional[str] = None,
        env: Optional[List[k8s_schemas.V1EnvVar]] = None,
        env_from: Optional[List[k8s_schemas.V1EnvFromSource]] = None,
        volume_mounts: Optional[List[k8s_schemas.V1VolumeMount]] = None,
        ports: Optional[List[k8s_schemas.V1ContainerPort]] = None,
        resources: Optional[k8s_schemas.V1ResourceRequirements] = None,
    ) -> k8s_schemas.V1Container:
        container.name = sanitize_container_name(name or container.name)
        container.env = to_list(container.env, check_none=True) + to_list(
            env, check_none=True
        )
        container.env_from = to_list(container.env_from, check_none=True) + to_list(
            env_from, check_none=True
        )
        container.volume_mounts = to_list(
            container.volume_mounts, check_none=True
        ) + to_list(volume_mounts, check_none=True)
        container.ports = to_list(container.ports, check_none=True) + to_list(
            ports, check_none=True
        )
        container.resources = container.resources or resources
        container._image_pull_policy = container.image_pull_policy or image_pull_policy
        container.image = container.image or image

        if not any([container.command, container.args]):
            container.command = command
            container.args = args

        return cls._sanitize_container(container)

    @staticmethod
    def _sanitize_container_env(
        env: List[k8s_schemas.V1EnvVar],
    ) -> Optional[List[k8s_schemas.V1EnvVar]]:
        def sanitize_env_dict(d: Dict):
            return {
                d_k: sanitize_value(d_v, handle_dict=False)
                if d_k in ["name", "value"]
                else sanitize_value(d_v, handle_dict=True)
                for d_k, d_v in d.items()
            }

        results = []
        for e in env or []:
            if isinstance(e, dict):
                e = sanitize_env_dict(e)
                results.append(e)
            elif isinstance(e, k8s_schemas.V1EnvVar):
                if e.value is not None:
                    e.value = sanitize_value(e.value, handle_dict=False)
                results.append(e)
        return results

    @staticmethod
    def _sanitize_resources(
        resources: Union[k8s_schemas.V1ResourceRequirements, Dict]
    ) -> Optional[k8s_schemas.V1ResourceRequirements]:
        def validate_resources(r_field: Dict) -> Dict:
            if not r_field:
                return r_field

            for k in r_field:
                r_field[k] = str(r_field[k])

            return r_field

        if not resources:
            return None

        if isinstance(resources, Dict):
            return k8s_schemas.V1ResourceRequirements(
                limits=validate_resources(resources.get("limits", None)),
                requests=validate_resources(resources.get("requests", None)),
            )
        else:
            return k8s_schemas.V1ResourceRequirements(
                limits=validate_resources(resources.limits),
                requests=validate_resources(resources.requests),
            )

    @classmethod
    def _sanitize_container(
        cls,
        container: k8s_schemas.V1Container,
    ) -> k8s_schemas.V1Container:
        container = sanitize_container_command_args(container)
        container.resources = cls._sanitize_resources(container.resources)
        container.env = cls._sanitize_container_env(container.env)
        return container



from typing import Any, Dict, List, Optional, Union
from typing_extensions import Literal

from clipped.types.ref_or_obj import RefField
from pydantic import Field

from polyaxon.k8s.k8s_schemas import V1Container
from polyaxon.polyflow.run.base import BaseRun
from polyaxon.polyflow.run.kinds import V1RunKind
from polyaxon.polyflow.run.ray.replica import V1RayReplica
from polyaxon.polyflow.run.resources import V1RunResources
from polyaxon.polyflow.run.utils import DestinationImageMixin


class V1RayJob(BaseRun, DestinationImageMixin):
    """Ray jobs are used to run Ray applications on Kubernetes.

    [Ray](https://www.ray.io/) Ray is an open-source unified compute framework that makes
    it easy to scale AI and Python workloads,
    from reinforcement learning to deep learning to tuning, and model serving.

    Args:
        kind: str, should be equal `rayjob`
        entrypoint: str, optional
        runtime_env: Dict, optional
        metadata: int, Dict, optional
        ray_version: str, optional
        head: [V1RayReplica](/docs/experimentation/distributed/ray-replica/), optional
        workers: Dict[str, [V1RayReplica](/docs/experimentation/distributed/ray-replica/)], optional


    ## YAML usage

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   entrypoint:
    >>>   runtimeRnv:
    >>>   metadata:
    >>>   rayVersion:
    >>>   head:
    >>>   workers:
    ```

    ## Python usage

    ```python
    >>> from polyaxon.polyflow import V1Environment, V1Init, V1RayJob, V1RayReplica
    >>> from polyaxon.k8s import k8s_schemas
    >>> ray_job = V1Ray(
    >>>     connections=["connection-name1"],
    >>>     volumes=[k8s_schemas.V1Volume(...)],
    >>>     ray_version="2.5.0",
    >>>     head=V1RayReplica(...),
    >>>     worker=V1RayReplica(...),
    >>> )
    ```

    ## Fields

    ### kind

    The kind signals to the CLI, client, and other tools that this component's runtime is a job.

    If you are using the python client to create the runtime,
    this field is not required and is set by default.

    ```yaml
    >>> run:
    >>>   kind: rayjob
    ```

    ### entrypoint

    The entrypoint command for this job.

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   entrypoint: python train.py
    ```

    ### runtimeEnv

    The runtime environment for this job.

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   runtimeEnv:
    >>>     pip: ["requests==2.26.0", "pendulum==2.1.2"]
    >>>     env_vars: {"counter_name": "test_counter"}
    ```

    ### rayVersion

    The version of Ray the application uses.

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   rayVersion: 2.5.0
    >>>   ...
    ```

    ### head

    Ray head replica specification

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   head:
    >>>     rayStartParams:
    >>>       dashboard-host: '0.0.0.0'
    >>>     container:
    >>>     ...
    >>>   ...
    ```

    ### workers

    List of worker replica specifications

    ```yaml
    >>> run:
    >>>   kind: rayjob
    >>>   workers:
    >>>     small-group:
    >>>       replicas: 1
    >>>       minReplicas: 1
    >>>       maxReplicas: 5
    >>>     ...
    >>>   ...
    ```
    """

    _IDENTIFIER = V1RunKind.RAYJOB

    kind: Literal[_IDENTIFIER] = _IDENTIFIER
    entrypoint: Optional[str]
    runtime_env: Optional[Union[Dict[str, Any], RefField]] = Field(alias="runtimeEnv")
    metadata: Optional[Union[Dict[str, str], RefField]]
    ray_version: Optional[str] = Field(alias="rayVersion")
    head: Optional[Union[V1RayReplica, RefField]]
    workers: Optional[Dict[str, Union[V1RayReplica, RefField]]]

    def apply_image_destination(self, image: str):
        if self.head:
            self.head.container = self.head.container or V1Container()
            self.head.container.image = image
        if self.workers:
            for worker_name in self.workers:
                worker = self.workers[worker_name]
                worker.container = worker.container or V1Container()
                worker.container.image = image

    def get_resources(self):
        resources = V1RunResources()
        if self.head:
            resources += self.head.get_resources()
        if self.workers:
            for worker_name in self.workers:
                worker = self.workers[worker_name]
                resources += worker.get_resources()
        return resources

    def get_all_containers(self):
        containers = []
        if self.head:
            containers += self.head.get_all_containers()
        if self.workers:
            for worker_name in self.workers:
                worker = self.workers[worker_name]
                containers += worker.get_all_containers()
        return containers

    def get_all_connections(self):
        connections = []
        if self.head:
            connections += self.head.get_all_connections()
        if self.workers:
            for worker_name in self.workers:
                worker = self.workers[worker_name]
                connections += worker.get_all_connections()
        return connections

    def get_all_init(self):
        init = []
        if self.head:
            init += self.head.get_all_init()
        if self.workers:
            for worker_name in self.workers:
                worker = self.workers[worker_name]
                init += worker.get_all_init()
        return init



import os
import time

import click

from clipped.formatting import Printer
from clipped.utils.lists import to_list
from clipped.utils.validation import validate_tags

from polyaxon.cli.dashboard import get_dashboard, get_dashboard_url
from polyaxon.cli.errors import handle_cli_error
from polyaxon.logger import clean_outputs
from polyaxon.managers.deploy import DeployConfigManager
from polyaxon.utils.fqn_utils import get_resource_name


def read_deployment_config(filepaths, command: str):
    from polyaxon.deploy import reader

    if not filepaths:
        return None

    filepaths = to_list(filepaths)
    for filepath in filepaths:
        if not os.path.isfile(filepath):
            Printer.error(
                "`{}` must be a valid file".format(filepath),
                sys_exit=True,
                command_help="polyaxon admin {}".format(command),
            )
    try:
        deployment_config = reader.read(filepaths)
        return deployment_config
    except Exception as e:
        handle_cli_error(
            e, message="Polyaxon deployment file is not valid", sys_exit=True
        )


@click.group()
@clean_outputs
def admin():
    """Commands for admin management."""


@admin.command()
@click.option(
    "-f",
    "--file",
    "config_file",
    type=click.Path(exists=True),
    help="The polyaxon deployment config file(s) to check.",
)
@click.option(
    "-t",
    "--deployment-type",
    help="Deployment type.",
)
@click.option(
    "--manager-path",
    type=click.Path(exists=True),
    help="The path of the deployment manager, e.g. local chart.",
)
@click.option(
    "--check",
    is_flag=True,
    default=False,
    help="Check if deployment file and other requirements are met.",
)
@click.option(
    "--dry-run",
    is_flag=True,
    default=False,
    help="Dry run the configuration and generate a debuggable output.",
)
@clean_outputs
def deploy(config_file, deployment_type, manager_path, check, dry_run):
    """Deploy polyaxon."""
    config = read_deployment_config(config_file, command="deploy")
    manager = DeployConfigManager(
        config=config,
        filepath=config_file,
        deployment_type=deployment_type,
        manager_path=manager_path,
        dry_run=dry_run,
    )
    exception = None
    if config:
        Printer.success(
            "Polyaxon `{}` deployment file is valid".format(config.deployment_chart)
        )
    if check:
        try:
            manager.check()
        except Exception as e:
            handle_cli_error(
                e, message="Polyaxon deployment manager error", sys_exit=True
            )

    else:
        try:
            manager.install()
        except Exception as e:
            Printer.error("Polyaxon could not be installed")
            exception = e

    if exception:
        Printer.error("Error message: {}".format(exception), sys_exit=True)


@admin.command()
@click.option(
    "-f",
    "--file",
    "config_file",
    type=click.Path(exists=True),
    help="The polyaxon deployment config file(s) to check.",
)
@click.option(
    "-t",
    "--deployment-type",
    help="Deployment type.",
)
@click.option(
    "--manager-path",
    type=click.Path(exists=True),
    help="The path of the deployment manager, e.g. local chart.",
)
@click.option(
    "--check",
    is_flag=True,
    default=False,
    help="Check if deployment file and other requirements are met.",
)
@click.option(
    "--dry-run",
    is_flag=True,
    default=False,
    help="Dry run the configuration and generate a debuggable output.",
)
@clean_outputs
def upgrade(config_file, deployment_type, manager_path, check, dry_run):
    """Upgrade a Polyaxon deployment."""
    config = read_deployment_config(config_file, command="upgrade")
    manager = DeployConfigManager(
        config=config,
        filepath=config_file,
        deployment_type=deployment_type,
        manager_path=manager_path,
        dry_run=dry_run,
    )
    exception = None
    if config:
        Printer.success(
            "Polyaxon `{}` deployment file is valid".format(config.deployment_chart)
        )
    if check:
        try:
            manager.check()
        except Exception as e:
            handle_cli_error(
                e, message="Polyaxon deployment manager error", sys_exit=True
            )
    else:
        try:
            manager.upgrade()
        except Exception as e:
            Printer.error("Polyaxon could not upgrade the deployment")
            exception = e

    if exception:
        Printer.error("Error message: {}".format(exception))


@admin.command()
@click.option(
    "-f",
    "--file",
    "config_file",
    type=click.Path(exists=True),
    help="The polyaxon deployment config file(s) to check.",
)
@click.option(
    "--manager-path",
    type=click.Path(exists=True),
    help="The path of the deployment manager, e.g. local chart.",
)
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="Automatic yes to prompts. "
    'Assume "yes" as answer to all prompts and run non-interactively.',
)
@clean_outputs
def teardown(config_file, manager_path, yes):
    """Teardown a polyaxon deployment given a config file."""
    config = read_deployment_config(config_file, command="teardown")
    manager = DeployConfigManager(
        config=config, filepath=config_file, manager_path=manager_path
    )
    exception = None
    try:
        if yes or click.confirm(
            "Would you like to execute pre-delete hooks?", default=False
        ):
            manager.teardown(hooks=True)
        else:
            manager.teardown(hooks=False)
    except Exception as e:
        Printer.error("Polyaxon could not teardown the deployment")
        exception = e

    if exception:
        Printer.error("Error message: {}".format(exception))


@admin.command()
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="Automatic yes to prompts. "
    'Assume "yes" as answer to all prompts and run non-interactively.',
)
@click.option(
    "--url", is_flag=True, default=False, help="Print the url of the dashboard."
)
@clean_outputs
def dashboard(yes, url):
    """Open dashboard in browser."""
    get_dashboard(dashboard_url=get_dashboard_url(base="_admin"), url_only=url, yes=yes)


@admin.command()
@click.option("--namespace", "-n", type=str)
@click.option("--in-cluster", is_flag=True, default=False)
@click.option("--delete", "-d", is_flag=True, default=False)
@click.option(
    "--uuids",
    "--uids",
    type=str,
    help="List of uuid of operations to clean/delete (comma separated values).",
)
def clean_ops(namespace, in_cluster, delete, uuids):
    """clean-ops command."""
    from polyaxon.k8s.custom_resources import operation
    from polyaxon.k8s.manager.manager import K8sManager

    if not namespace:
        raise Printer.error("The argument `--namespace` is required!", sys_exit=True)

    manager = K8sManager(namespace=namespace, in_cluster=in_cluster)

    def _patch_op():
        retry = 0
        while retry < 3:
            if retry:
                time.sleep(retry * 2)
            try:
                manager.update_custom_object(
                    name=op,
                    group=operation.GROUP,
                    version=operation.API_VERSION,
                    plural=operation.PLURAL,
                    body={"metadata": {"finalizers": None}},
                )
                return
            except Exception as e:
                print("Exception %s", e)
                print("retrying")
                retry += 1

    def _delete_op():
        retry = 0
        while retry <= 2:
            if retry:
                time.sleep(retry)
            try:
                manager.delete_custom_object(
                    name=op,
                    group=operation.GROUP,
                    version=operation.API_VERSION,
                    plural=operation.PLURAL,
                )
                return
            except Exception as e:
                print("Exception %s", e)
                print("retrying")
                retry += 1

    uuids = validate_tags(uuids, validate_yaml=True)
    if uuids:
        ops = [o if "plx-operation-" in o else get_resource_name(o) for o in uuids]
    else:
        ops = [
            o["metadata"]["name"]
            for o in manager.list_custom_objects(
                group=operation.GROUP,
                version=operation.API_VERSION,
                plural=operation.PLURAL,
            )
        ]
    if not ops:
        return

    Printer.header(f"Cleaning {len(ops)} ops ...")
    for idx, op in enumerate(ops):
        with Printer.console.status(f"Cleaning operation {idx + 1}/{len(ops)} ..."):
            _patch_op()
            if delete:
                _delete_op()
        Printer.success(f"Operation {op} was cleaned")



import sys

from clipped.formatting import Printer
from clipped.utils.strings import validate_slug

from polyaxon.constants.globals import DEFAULT
from polyaxon.env_vars.getters.user import get_local_owner
from polyaxon.exceptions import PolyaxonClientException, PolyaxonSchemaError
from polyaxon.utils.fqn_utils import get_entity_info


def resolve_entity_info(entity: str, entity_name: str, is_cli: bool = False):
    from polyaxon import settings

    if not entity:
        message = "Please provide a valid {}!".format(entity_name)
        if is_cli:
            Printer.error(message)
            sys.exit(1)
        else:
            raise PolyaxonClientException(message)

    owner, entity_value = get_entity_info(entity)

    if not owner:
        owner = get_local_owner(is_cli=is_cli)

    if not owner and (not settings.CLI_CONFIG or settings.CLI_CONFIG.is_community):
        owner = DEFAULT

    if not owner:
        owner = settings.AUTH_CONFIG.username if settings.AUTH_CONFIG else None

    if not all([owner, entity_value]):
        message = "Please provide a valid {}.".format(entity_name)
        if is_cli:
            Printer.error(message)
            sys.exit(1)
        else:
            raise PolyaxonClientException(message)
    if owner and not validate_slug(owner):
        raise PolyaxonSchemaError(
            "Received an invalid owner, received the value: `{}`".format(owner)
        )

    if entity_value and not validate_slug(entity_value):
        raise PolyaxonSchemaError(
            "Received an invalid {}, received the value: `{}`".format(
                entity_name, entity_value
            )
        )
    return owner, entity_value



from typing import List, Optional, Union

from clipped.types.ref_or_obj import BoolOrRef, FloatOrRef, RefField
from clipped.utils.lists import to_list
from pydantic import Field, StrictStr, constr, validator

from polyaxon.polyflow.builds import V1Build
from polyaxon.polyflow.cache import V1Cache
from polyaxon.polyflow.hooks import V1Hook
from polyaxon.polyflow.plugins import V1Plugins
from polyaxon.polyflow.termination import V1Termination
from polyaxon.schemas.base import NAME_REGEX, BaseSchemaModel


class BaseComponent(BaseSchemaModel):
    version: Optional[float]
    kind: Optional[StrictStr]
    name: Optional[Union[constr(regex=NAME_REGEX), RefField]]
    description: Optional[StrictStr]
    tags: Optional[List[StrictStr]]
    presets: Optional[List[StrictStr]]
    queue: Optional[StrictStr]
    cache: Optional[Union[V1Cache, RefField]]
    termination: Optional[Union[V1Termination, RefField]]
    plugins: Optional[Union[V1Plugins, RefField]]
    build: Optional[Union[V1Build, RefField]]
    hooks: Optional[Union[List[V1Hook], RefField]]
    is_approved: Optional[BoolOrRef] = Field(alias="isApproved")
    cost: Optional[FloatOrRef]

    @validator("tags", "presets", pre=True)
    def validate_str_list(cls, v):
        if isinstance(v, str):
            return to_list(v, check_str=True)
        return v



from typing import Dict, List, Optional

from pydantic import Extra, Field, StrictStr

from polyaxon.schemas.base import BaseSchemaModel


class IngressConfig(BaseSchemaModel):
    enabled: Optional[bool]
    host_name: Optional[StrictStr] = Field(alias="hostName")
    path: Optional[StrictStr]
    tls: Optional[List[Dict]]
    annotations: Optional[Dict]

    class Config:
        extra = Extra.ignore



from typing import Optional

from pydantic import StrictStr

from polyaxon.schemas.types.base import BaseTypeConfig


class V1EventType(BaseTypeConfig):
    _IDENTIFIER = "event"

    name: Optional[StrictStr]
    kind: Optional[StrictStr]



from typing import List, Optional

from polyaxon.schemas.base import BaseResponseModel
from polyaxon.schemas.responses.v1_section_spec import V1SectionSpec


class V1DashboardSpec(BaseResponseModel):
    sections: Optional[List[V1SectionSpec]]



import os

from typing import Dict, List, Optional

from clipped.config.schema import skip_partial, to_partial
from pydantic import Extra, Field, StrictStr, root_validator, validator
from vents.connections import ConnectionCatalog

from polyaxon.auxiliaries import (
    V1DefaultScheduling,
    V1PolyaxonCleaner,
    V1PolyaxonInitContainer,
    V1PolyaxonNotifier,
    V1PolyaxonSidecarContainer,
)
from polyaxon.config.parser import ConfigParser
from polyaxon.connections import V1Connection, V1ConnectionKind, V1HostPathConnection
from polyaxon.contexts import paths as ctx_paths
from polyaxon.env_vars.getters import get_artifacts_store_name
from polyaxon.env_vars.keys import (
    EV_KEYS_AGENT_ARTIFACTS_STORE,
    EV_KEYS_AGENT_CLEANER,
    EV_KEYS_AGENT_CONNECTIONS,
    EV_KEYS_AGENT_DEFAULT_IMAGE_PULL_SECRETS,
    EV_KEYS_AGENT_DEFAULT_SCHEDULING,
    EV_KEYS_AGENT_EXECUTOR_REFRESH_INTERVAL,
    EV_KEYS_AGENT_INIT,
    EV_KEYS_AGENT_IS_REPLICA,
    EV_KEYS_AGENT_NOTIFIER,
    EV_KEYS_AGENT_RUNS_SA,
    EV_KEYS_AGENT_SECRET_NAME,
    EV_KEYS_AGENT_SIDECAR,
    EV_KEYS_AGENT_USE_PROXY_ENV_VARS_IN_OPS,
    EV_KEYS_ARTIFACTS_STORE_NAME,
    EV_KEYS_K8S_APP_SECRET_NAME,
    EV_KEYS_K8S_NAMESPACE,
)
from polyaxon.exceptions import PolyaxonSchemaError
from polyaxon.fs.utils import get_store_path
from polyaxon.schemas.base import BaseSchemaModel


def validate_agent_config(
    artifacts_store, connections, required_artifacts_store: bool = True
) -> None:
    if required_artifacts_store and not artifacts_store:
        raise PolyaxonSchemaError(
            "A connection definition is required to set a default artifacts store."
        )

    connections = connections or []

    connection_names = set()

    if artifacts_store:
        connection_names.add(artifacts_store.name)

    for c in connections:
        if c.name in connection_names:
            raise PolyaxonSchemaError(
                "A connection with name `{}` must be unique.".format(c.name)
            )
        connection_names.add(c.name)


class BaseAgentConfig(ConnectionCatalog, BaseSchemaModel):
    _REQUIRED_ARTIFACTS_STORE = True

    connections: Optional[List[V1Connection]] = Field(alias=EV_KEYS_AGENT_CONNECTIONS)
    artifacts_store: Optional[V1Connection] = Field(alias=EV_KEYS_AGENT_ARTIFACTS_STORE)
    namespace: Optional[StrictStr] = Field(alias=EV_KEYS_K8S_NAMESPACE)

    class Config:
        extra = Extra.ignore

    @root_validator(pre=True)
    def handle_camel_case_artifacts_store(cls, values):
        if (
            not values.get("artifacts_store")
            and not values.get(EV_KEYS_AGENT_ARTIFACTS_STORE)
            and "artifactsStore" in values
        ):
            values[EV_KEYS_AGENT_ARTIFACTS_STORE] = values["artifactsStore"]
        return values

    @validator("connections", pre=True)
    def validate_json_list(cls, v):
        if not isinstance(v, str):
            return v
        try:
            return ConfigParser.parse(Dict)(
                key=EV_KEYS_AGENT_CONNECTIONS,
                value=v,
                is_list=True,
                is_optional=True,
            )
        except PolyaxonSchemaError as e:
            raise ValueError("Received an invalid connections") from e

    @validator("artifacts_store", pre=True)
    def validate_store_json(cls, v):
        if not isinstance(v, str):
            return v
        try:
            return ConfigParser.parse(Dict)(
                key=EV_KEYS_AGENT_ARTIFACTS_STORE,
                value=v,
                is_optional=True,
            )
        except PolyaxonSchemaError as e:
            raise ValueError(
                "Received an invalid artifacts store `{}`".format(v)
            ) from e

    @validator("artifacts_store")
    @skip_partial
    def validate_agent_config(cls, artifacts_store, values):
        try:
            validate_agent_config(
                artifacts_store,
                values.get("connections"),
                cls._REQUIRED_ARTIFACTS_STORE,
            )
        except PolyaxonSchemaError as e:
            raise ValueError(e)
        return artifacts_store

    def set_all_connections(self) -> None:
        self._all_connections = self.connections[:] if self.connections else []
        self._connections_by_names = {}
        if self.artifacts_store:
            self._all_connections.append(self.artifacts_store)
            validate_agent_config(
                self.artifacts_store, self.connections, self._REQUIRED_ARTIFACTS_STORE
            )

    @property
    def local_root(self) -> str:
        artifacts_root = ctx_paths.CONTEXT_ARTIFACTS_ROOT
        if not self.artifacts_store:
            return artifacts_root

        if self.artifacts_store.is_mount:
            return self.artifacts_store.store_path

        return artifacts_root

    def get_local_path(self, subpath: str, entity: Optional[str] = None) -> str:
        return get_store_path(
            store_path=self.local_root, subpath=subpath, entity=entity
        )

    @property
    def store_root(self) -> str:
        if not self.artifacts_store:
            return ctx_paths.CONTEXT_ARTIFACTS_ROOT

        return self.artifacts_store.store_path

    def get_store_path(self, subpath: str, entity: Optional[str] = None) -> str:
        return get_store_path(
            store_path=self.store_root, subpath=subpath, entity=entity
        )

    def set_default_artifacts_store(self):
        if not self.artifacts_store:
            self.artifacts_store = V1Connection(
                name=get_artifacts_store_name(),
                kind=V1ConnectionKind.HOST_PATH,
                schema_=V1HostPathConnection(
                    host_path=self.store_root, mount_path=self.store_root
                ),
            )
            self.set_all_connections()

    def set_artifacts_store_name(self):
        if self.artifacts_store:
            os.environ[EV_KEYS_ARTIFACTS_STORE_NAME] = self.artifacts_store.name


class AgentConfig(BaseAgentConfig):
    _IDENTIFIER = "agent"

    is_replica: Optional[bool] = Field(alias=EV_KEYS_AGENT_IS_REPLICA)
    sidecar: Optional[V1PolyaxonSidecarContainer] = Field(alias=EV_KEYS_AGENT_SIDECAR)
    init: Optional[V1PolyaxonInitContainer] = Field(alias=EV_KEYS_AGENT_INIT)
    notifier: Optional[V1PolyaxonNotifier] = Field(alias=EV_KEYS_AGENT_NOTIFIER)
    cleaner: Optional[V1PolyaxonCleaner] = Field(alias=EV_KEYS_AGENT_CLEANER)
    use_proxy_env_vars_use_in_ops: Optional[bool] = Field(
        alias=EV_KEYS_AGENT_USE_PROXY_ENV_VARS_IN_OPS
    )
    default_scheduling: Optional[V1DefaultScheduling] = Field(
        alias=EV_KEYS_AGENT_DEFAULT_SCHEDULING
    )
    default_image_pull_secrets: Optional[List[StrictStr]] = Field(
        alias=EV_KEYS_AGENT_DEFAULT_IMAGE_PULL_SECRETS
    )
    app_secret_name: Optional[StrictStr] = Field(alias=EV_KEYS_K8S_APP_SECRET_NAME)
    agent_secret_name: Optional[StrictStr] = Field(alias=EV_KEYS_AGENT_SECRET_NAME)
    runs_sa: Optional[StrictStr] = Field(alias=EV_KEYS_AGENT_RUNS_SA)
    # This refresh logic will mitigate several issues with AKS's numerous networking problems
    executor_refresh_interval: Optional[int] = Field(
        alias=EV_KEYS_AGENT_EXECUTOR_REFRESH_INTERVAL
    )

    @root_validator(pre=True)
    def handle_camel_case_agent(cls, values):
        if (
            not values.get("is_replica")
            and not values.get(EV_KEYS_AGENT_IS_REPLICA)
            and "isReplica" in values
        ):
            values[EV_KEYS_AGENT_IS_REPLICA] = values["isReplica"]
        if (
            not values.get("use_proxy_env_vars_use_in_ops")
            and not values.get(EV_KEYS_AGENT_USE_PROXY_ENV_VARS_IN_OPS)
            and "useProxyEnvVarsUseInOps" in values
        ):
            values[EV_KEYS_AGENT_USE_PROXY_ENV_VARS_IN_OPS] = values[
                "useProxyEnvVarsUseInOps"
            ]
        if (
            not values.get("default_scheduling")
            and not values.get(EV_KEYS_AGENT_DEFAULT_SCHEDULING)
            and "defaultScheduling" in values
        ):
            values[EV_KEYS_AGENT_DEFAULT_SCHEDULING] = values["defaultScheduling"]
        if (
            not values.get("default_image_pull_secrets")
            and not values.get(EV_KEYS_AGENT_DEFAULT_IMAGE_PULL_SECRETS)
            and "defaultImagePullSecrets" in values
        ):
            values[EV_KEYS_AGENT_DEFAULT_IMAGE_PULL_SECRETS] = values[
                "defaultImagePullSecrets"
            ]
        if (
            not values.get("app_secret_name")
            and not values.get(EV_KEYS_K8S_APP_SECRET_NAME)
            and "appSecretName" in values
        ):
            values[EV_KEYS_K8S_APP_SECRET_NAME] = values["appSecretName"]
        if (
            not values.get("agent_secret_name")
            and not values.get(EV_KEYS_AGENT_SECRET_NAME)
            and "agentSecretName" in values
        ):
            values[EV_KEYS_AGENT_SECRET_NAME] = values["agentSecretName"]
        if (
            not values.get("runs_sa")
            and not values.get(EV_KEYS_AGENT_RUNS_SA)
            and "runsSa" in values
        ):
            values[EV_KEYS_AGENT_RUNS_SA] = values["runsSa"]
        if (
            not values.get("executor_refresh_interval")
            and not values.get(EV_KEYS_AGENT_EXECUTOR_REFRESH_INTERVAL)
            and "executorRefreshInterval" in values
        ):
            values[EV_KEYS_AGENT_EXECUTOR_REFRESH_INTERVAL] = values[
                "executorRefreshInterval"
            ]
        return values

    def __init__(
        self,
        default_scheduling=None,
        default_image_pull_secrets=None,
        **data,
    ):
        if not default_scheduling and default_image_pull_secrets:
            default_scheduling = V1DefaultScheduling()
        if default_scheduling and not default_scheduling.image_pull_secrets:
            default_scheduling.image_pull_secrets = default_image_pull_secrets
        super().__init__(
            default_scheduling=default_scheduling,
            default_image_pull_secrets=default_image_pull_secrets,
            **data,
        )

    @validator(
        "artifacts_store",
        "sidecar",
        "init",
        "cleaner",
        "notifier",
        "default_scheduling",
        pre=True,
    )
    def validate_json(cls, v, field):
        if not isinstance(v, str):
            return v
        try:
            return ConfigParser.parse(Dict)(
                key=field.name,
                value=v,
                is_optional=True,
            )
        except PolyaxonSchemaError as e:
            raise ValueError(
                "Received an invalid {} `{}`".format(field.alias, v)
            ) from e

    @validator("default_image_pull_secrets", pre=True)
    def validate_str_list(cls, v, field):
        try:
            return ConfigParser.parse(str)(
                key=field.alias,
                value=v,
                is_optional=True,
                is_list=True,
            )
        except PolyaxonSchemaError as e:
            raise ValueError(
                "Received an invalid {} `{}`".format(field.alias, v)
            ) from e

    def get_executor_refresh_interval(self) -> int:
        return self.executor_refresh_interval or 60 * 5


PartialAgentConfig = to_partial(AgentConfig)
